@@ -8,7 +8,9 @@ use MIME::Base64 qw(decode_base64 encode_base64);
 use Data::GUID;
 use Carp qw/croak/;
 
-our $VERSION = '0.01';
+use parent 'Authen::Passphrase';
+
+our $VERSION = '0.02';
 
 our (%salts, %hashes);
 BEGIN {
@@ -48,7 +50,7 @@ BEGIN {
 }
 
 sub new {
-	my ($class, %args) = @_;
+	my ($class, %args) = (shift, (scalar @_ > 1 ? @_ : %{$_[0]}));
 	my $self = bless({ algorithm => 'Argon2' }, $class);
 	my $passphrase = $args{passphrase};
 	for (keys %salts) {
@@ -60,7 +62,7 @@ sub new {
 	for (qw/stored_hash stored_base64 stored_hex/) {
 		if ($args{$_}) {
 			my @change = split "_", $_;
-			$args{'base_' . $change[1]} = $passphrase;
+			$args{'hash_' . $change[1]} = $passphrase;
 			$passphrase = undef;
 			last;
 		}
@@ -70,7 +72,6 @@ sub new {
 			croak "hash specified redundantly"
 				if exists($self->{hash}) || defined($passphrase);
 			$self->{crypt} = $hashes{$_}->($args{$_});
-			$passphrase = '';
 		}
 	}
 	croak "salt not specified" unless exists $self->{salt};
@@ -89,7 +90,7 @@ sub _hash_of {
 	if ($pass =~ m/\$argon2/) {
 		return $pass;
 	}
-	
+
 	return argon2i_pass($pass, $self->{salt}, $self->{cost}, $self->{factor}, $self->{parallelism}, $self->{size});
 }
 
@@ -153,7 +154,8 @@ sub match {
 }
 
 sub from_crypt {
-	return $_[1];
+	my ($self, $passphrase, $info) = @_;
+	return $self->new({ %{ $info },  passphrase => $passphrase });
 }
 
 1;
@@ -166,7 +168,7 @@ Authen::Passphrase::Argon2 - Store and check password using Argon2
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
@@ -176,7 +178,7 @@ Version 0.01
 
 	$ppr = Authen::Passphrase::Argon2->new(
 		salt_random => 1,
-		hash_hex => '246172676f6e326924763d3139246d3d33323736382c743d332c703d312459574a6a5a47566d5a7a45794d7724435348383332634a6330347376516f7656492f754441', 
+		hash_hex => '246172676f6e326924763d3139246d3d33323736382c743d332c703d312459574a6a5a47566d5a7a45794d7724435348383332634a6330347376516f7656492f754441',
 	);
 
 	$algorithm = $ppr->algorithm;
@@ -185,7 +187,7 @@ Version 0.01
 	$hash = $ppr->hash;
 	$hash_hex = $ppr->hash_hex;
 
-	if($ppr->match($passphrase)) { 
+	if($ppr->match($passphrase)) {
 		...
 	}
 
@@ -194,7 +196,7 @@ Version 0.01
 	.....
 
 	__PACKAGE__->add_columns(
-	
+
 	);
 
 =cut
@@ -215,7 +217,7 @@ The term **hash** is loosely used in this documentation and implementation. Real
 params
 
 	# Only one of these must be defined for the salt value. An error will be thrown if none or more than one is defined.
-	salt - plain text salt value 
+	salt - plain text salt value
 	salt_hex - salt in hex value that gets decoded.
 	salt_base64 - salt value encoded in base64 that gets decoded.
 	salt_random - a random salt will be generated using Data::GUID.
@@ -229,7 +231,7 @@ params
 	# optional params
 	cost - optional - default 3 - This is the time-cost factor, typically a small integer that can be derived as explained above.
 	factor - optional - default '32M' - This is the memory costs factor. This must be given as a integer followed by an order of magnitude (k, M or G for kilobytes, megabytes or gigabytes respectively), e.g. '64M'.
-	parallelism - optional - default 1 - This is the number of threads that are used in computing it. 
+	parallelism - optional - default 1 - This is the number of threads that are used in computing it.
 	size - optional - default 16 - This is the size of the raw result in bytes. Typical values are 16 or 32.
 
 =cut
@@ -276,17 +278,11 @@ Returns the raw argon2 formatted string encdoed in base64.
 
 =cut
 
-=head2 $ppr->as_base64
-
-Returns the raw argon2 formatted string encdoed in base64.
-
-=cut
-
 =head2 $ppr->hash
 
 Returns the hash value, in raw form.
 
-(lets keep consistency but it's just an alias for as_crypt, perhaps I have some misunderstanding /o\) 
+(lets keep consistency but it's just an alias for as_crypt, perhaps I have some misunderstanding /o\)
 
 =cut
 
@@ -296,7 +292,7 @@ Returns the hash value, as a string of hexadecimal digits.
 
 =cut
 
-=head2 $ppr->hash_base64 
+=head2 $ppr->hash_base64
 
 Returns the hash value, as a string in base64.
 
